==> comphep-4.4.113.tgz: comphep-4.4.113/src/num/mix_lhef.c <==
/*
* Copyright (C) 2007-2008, CompHEP Collaboration
* Author: Alexander Sherstnev 
* ------------------------------------------------------
* $Id$
*
* $Log$
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include<stdio.h>
#include<stdlib.h>
#include<time.h>
#include<string.h>

#include "mix_format_lhef.h"
#include "lhef_routines.h"

static int mix_format_lhef (const int nf, const char names[], const int lenth)
{

  int i, j;
  int nRecotot;
  int nLefttot;
  int *nReco;
  int *nLeft;

  long endpos;
  long **pos;
  long *pos0;

  double *ri;
  double *CSec;
  double xrn;
  double sigmatot = 0.0;
  char **map;
  char **filename;

  FILE** infile;
  FILE* outFile;
  FILE* prtFile;

  char* outFileName = "Mixed.PEV";
  char* prtFileName = "prt.pev";

  filename = malloc (nf * sizeof(char*));
  infile   = malloc (nf * sizeof(FILE*));

  pos0  = malloc (nf * sizeof (long));
  pos   = malloc (nf * sizeof (long *));
  CSec  = malloc (nf * sizeof (double));
  ri    = malloc (nf * sizeof (double));
  map   = malloc (nf * sizeof (char *));
  nReco = malloc (nf * sizeof (int));
  nLeft = malloc (nf * sizeof (int));

  prtFile = fopen (prtFileName, "w");
  if (NULL == prtFile) {
    fprintf (stderr, "mix: Error!");
    fprintf (prtFile, "Files to mix and randomize:\n");
  }

  for (i = 0; i < nf; ++i) {
    filename[i] = malloc ((lenth + 1) * sizeof(char));
    strncpy (filename[i], names + i * lenth, lenth);
    trim (filename[i]);
    infile[i] = fopen (filename[i], "r");
    if (!infile[i]) {
      fprintf (stderr, "File not found: %s\n", filename[i]);
      fprintf (prtFile, "File not found: %s\n", filename[i]);
      return 2;
    } else {
      fprintf (prtFile, "%s\n", filename[i]);
    }
    fclose (infile[i]);
  }

  nLefttot = 0;
  sigmatot = 0.0;
  for (i = 0; i < nf; ++i) {
    int num = 0;
    formXMLdocument (filename[i], i);
    formXMLtree (filename[i], i);
    pos0[i]  = getEventPosition (i);
    nLeft[i] = getEventNumber (i);
    CSec[i]  = getCrossSection (i);
    infile[i] = fopen (filename[i], "r");
    nLefttot += nLeft[i];
    sigmatot += CSec[i];
    pos[i] = malloc (nLeft[i] * sizeof (long));
    map[i] = malloc (nLeft[i] * sizeof (char));
    pos[i][num] = pos0[i];
    map[i][num] = 0;
    while (!testEvent (infile[i], filename[i], pos[i][num])) {
      ++num;
      pos[i][num] = ftell (infile[i]);
      map[i][num] = 0;
    }
    if (num != nLeft[i]) {
      fprintf (stderr, "warning: the real number of event in file %s does not correspond reporting number in header.\n", filename[i]);
      nLeft[i] = num;
    }
  }

  xrn = 0.0;
  for (i = 0; i < nf; i++) {
    ri[i] = xrn + CSec[i] / sigmatot;
    xrn = ri[i];
    nReco[i] = 0;
  }

  endpos = write_event_header_libxml2 (outFileName, "w");
  outFile = fopen (outFileName, "a");

  nRecotot = 0;
  while (nRecotot < nLefttot) {
    xrn = drand48 ();
    i = 0;
    while (xrn > ri[i]) {
      i++;
    }

    if (nLeft[i] == nReco[i]) {
      int filesize = 0;
      double errtot = 0.0;
      char check_sum[128];

      fprintf (stdout,  "mix: file %s exhausted!\n", filename [i]);
      fprintf (stdout,  "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
      fprintf (prtFile, "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
      fclose (outFile);
      fclose (prtFile);
      {
        struct stat stt;
        stat (outFileName, &stt);
        filesize = stt.st_size;
      }
      {
        char command[1024];
        for (i + 0; i < 128; ++i) check_sum[i] = 0;
        sprintf (command, "md5sum %s > md5sum.log", outFileName);
        system(command);
        FILE * md5 = fopen ("md5sum.log", "r");
        if (md5) {
          fscanf (md5, "%s", check_sum);
        }
      }
      set_final_numbers (nRecotot, filesize, sigmatot, errtot, check_sum);
      write_event_header_libxml2 (outFileName, "r+");

      for (i = 0; i < nf; ++i) {
        fclose (infile[i]);
        free (filename[i]);
        free (pos[i]);
        free (map[i]);
      }
      free (filename);
      free (infile);
      free (pos);
      free (map);
      free (pos0);
      free (CSec);
      free (nReco);
      free (nLeft);
      free (ri);

      return 1;
    }

    j = 0;
    while (map[i][j]) {
      ++j;
    }
    moveEvent (infile[i], filename[i], pos[i][j], outFile, endpos);
    endpos = ftell(outFile);
    map[i][j] = 1;
    ++(nReco[i]);
    ++nRecotot;
  }

  fprintf (stdout,  "mix: all files exhausted\n");
  fprintf (stdout,  "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
  fprintf (prtFile, "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
  fclose (outFile);
  fclose (prtFile);
  {
    int filesize = 0;
    double errtot = 0.0;
    char check_sum[128];
    struct stat stt;
    char command[1024];

    stat (outFileName, &stt);
    filesize = stt.st_size;
    for (i + 0; i < 128; ++i) check_sum[i] = 0;
    sprintf (command, "md5sum %s > md5sum.log", outFileName);
    system(command);
    FILE * md5 = fopen ("md5sum.log", "r");
    if (md5) {
      fscanf (md5, "%s", check_sum);
    }
    set_final_numbers (nRecotot, filesize, sigmatot, errtot, check_sum);
    write_event_header_libxml2 (outFileName, "r+");
  }

  for (i = 0; i < nf; ++i) {
    fclose (infile[i]);
    free (filename[i]);
    free (pos[i]);
    free (map[i]);
  }
  free (filename);
  free (infile);
  free (pos);
  free (map);
  free (pos0);
  free (CSec);
  free (nReco);
  free (nLeft);
  free (ri);

  return 1;
}


int main (int argc, char** argv)
{
  int i;
  int nf = argc - 1;
  int frmt;
  int lenth = 0;
  char * names;

  if (argc < 2) {
    fprintf(stderr, "mix: Enter names of files with events as parameters. \n");
    return 1;
  }

  for (i = 0; i < nf; ++i) {
    int tlenth;
    FILE* inFile = fopen(argv[i + 1], "r");
    if (!inFile) {
      fprintf(stderr,"mix: error! File not found: %s\n", argv[i + 1]);
      return -2;
    } else {
      if (0 == i) frmt = CheckFormat (inFile);
      if (frmt != CheckFormat (inFile)) {
        fprintf(stderr, "mix: error! Different formats in the event files!\n");
        return -3;
      }
      tlenth = strlen (argv[i + 1]);
      if (lenth < tlenth) lenth = tlenth;
    }
    fclose (inFile);
  }

  names = malloc (nf * lenth * sizeof (char));
  for (i = 0; i < nf; ++i) {
    strcpy (names + i * lenth, argv[i + 1]);
  }

  if (4 == frmt) {
    fprintf (stdout, "mix: format lhe detected in all event files\n");
    mix_format_lhef (nf, names, lenth);
  } else {
    fprintf(stderr, "Error! Not LesHouche event format in event files!\n");
    return 4;
  }
  free (names);
  return 0;
}

==> comphep-4.5.0rc1.tgz: comphep-4.5.0rc1/src/num/mix_lhef.c <==
/*
* Copyright (C) 2007-2008, CompHEP Collaboration
* Author: Alexander Sherstnev 
* ------------------------------------------------------
* $Id$
*
* $Log$
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include<stdio.h>
#include<stdlib.h>
#include<time.h>
#include<string.h>

#include "service2/include/syst.h"

#include "evnt_tools.h"
#include "mix_format_lhef.h"
#include "lhef_routines.h"

int main (int argc, char** argv)
{
  int i;
  int nf = argc - 1;
  int frmt;
  int lenth = 0;
  char * names;

  if (argc < 2) {
    fprintf(stderr, "mix: Enter names of files with events as parameters. \n");
    return 1;
  }

  for (i = 0; i < nf; ++i) {
    int tlenth;
    FILE* inFile = fopen(argv[i + 1], "r");
    if (!inFile) {
      fprintf(stderr,"mix: error! File not found: %s\n", argv[i + 1]);
      return -2;
    } else {
      if (0 == i) frmt = CheckFormat (inFile);
      if (frmt != CheckFormat (inFile)) {
        fprintf(stderr, "mix: error! Different formats in the event files!\n");
        return -3;
      }
      tlenth = strlen (argv[i + 1]);
      if (lenth < tlenth) lenth = tlenth;
    }
    fclose (inFile);
  }

  names = malloc (nf * lenth * sizeof (char));
  for (i = 0; i < nf; ++i) {
    strcpy (names + i * lenth, argv[i + 1]);
  }

  if (4 == frmt) {
    fprintf (stdout, "mix: format lhe detected in all event files\n");
    mix_format_lhef (nf, names, lenth);
  } else {
    fprintf(stderr, "Error! Not LesHouches event format in event files!\n");
    return 4;
  }
  free (names);
  return 0;
}
